/*
 * Lesson 03: Adding Color
 * Inspired by the Nehe examples at nehe.gamedev.net
 */

import groovy.swing.SwingXBuilder
import com.sun.j3d.utils.universe.*
import javax.media.j3d.*
import javax.vecmath.*
import java.awt.event.*
import javax.swing.WindowConstants
import groovy.j3d.Java3DBuilder

/*
* @author jwill
*/



class Lesson03 {
    def swing = new SwingXBuilder()
    def j3d = new Java3DBuilder()
    def config = SimpleUniverse.getPreferredConfiguration();
    def canvas3D = new Canvas3D(config)
    def frame
    def keyListener = [keyPressed: {evt ->
        int keyCode = evt.getKeyCode()
        if (keyCode == KeyEvent.VK_ESCAPE)
            System.exit(0)
    }, keyReleased: {evt ->}, keyTyped: {evt ->}] as KeyListener

    void createGeometryAndView() {
        def su = j3d.simpleUniverse(canvas: canvas3D) {
            branchGroup {
                shape3D {
                    triangleArray(numVertices: 3, format: GeometryArray.COORDINATES | GeometryArray.COLOR_3) {
                        vertices {
                            point(x: 0.0f, y: 1.0f, z: 0.0f)
                            point(x: -1.0f, y: -1.0f, z: 0.0f)
                            point(x: 1.0f, y: -1.0f, z: 0.0f)
                        }
                        colors {
                            color(r: 1.0f, g: 0.0f, b: 0.0f)
                            color(r: 0.0f, g: 1.0f, b: 0.0f)
                            color(r: 0.0f, g: 0.0f, b: 1.0f)
                        }
                    }
                }
                transformGroup(translation: new Vector3f(3.0f, 0.0f, 0.0f)) {
                    shape3D {
                        quadArray(numVertices: 4, format: GeometryArray.COORDINATES | GeometryArray.COLOR_3) {
                            vertices {
                                point(x: -1.0f, y: -1.0f, z: 0.0f)
                                point(x: 1.0f, y: -1.0f, z: 0.0f)
                                point(x: 1.0f, y: 1.0f, z: 0.0f)
                                point(x: -1.0f, y: 1.0f, z: 0.0f)
                            }
                            colors {
                                color(r: 0.5f, g: 0.5f, b: 1.0f)
                                color(r: 0.5f, g: 0.5f, b: 1.0f)
                                color(r: 0.5f, g: 0.5f, b: 1.0f)
                            }
                        }
                    }
                }

            }
            viewTransformGroup(translation: new Vector3f(1.5f, 0.0f, 6.5f))
        }
    }

    void setupScene() {
        canvas3D.addKeyListener(keyListener)

        frame = swing.frame(size: [400, 200], title: 'Lesson 03: Adding Color', defaultCloseOperation: WindowConstants.EXIT_ON_CLOSE) {
            widget(canvas3D)
        }

        swing.doLater() {
            frame.show()
        }
    }

    public static void main(args) {
        def demo = new Lesson03()
        demo.createGeometryAndView()
        demo.setupScene()
    }
}