/*
* Lesson 04: Rotation
* Inspired by the Nehe examples at nehe.gamedev.net
*/

import com.sun.j3d.utils.universe.SimpleUniverse
import groovy.swing.SwingXBuilder
import java.awt.event.KeyEvent
import java.awt.event.KeyListener
import javax.media.j3d.*
import javax.swing.WindowConstants
import javax.vecmath.Color3f
import javax.vecmath.Point3f
import javax.vecmath.Vector3f
import org.jdesktop.animation.timing.Animator
import org.jdesktop.animation.timing.TimingTarget
import groovy.j3d.Java3DBuilder

/*
* @author jwill
*/

class Lesson04 implements TimingTarget {
    def j3d = new Java3DBuilder()
    def swing = new SwingXBuilder()
    def config = SimpleUniverse.getPreferredConfiguration();
    def canvas3D = new Canvas3D(config)
    def frame, transform, transform2
    def rtri = 0, rquad = 0
    def animator = new Animator(4000, Animator.INFINITE, Animator.RepeatBehavior.LOOP, this)
    def keyListener = [keyPressed: {evt ->
        int keyCode = evt.getKeyCode()
        if (keyCode == KeyEvent.VK_ESCAPE)
            System.exit(0)
    }, keyReleased: {evt ->}, keyTyped: {evt ->}] as KeyListener

    void createGeometryAndView() {
        def su = j3d.simpleUniverse(canvas: canvas3D) {
            branchGroup {
                transform = transformGroup(capability: TransformGroup.ALLOW_TRANSFORM_WRITE) {
                    shape3D {
                        triangleArray(numVertices: 3, format: GeometryArray.COORDINATES | GeometryArray.COLOR_3) {
                            vertices {
                                point(x: 0.0f, y: 1.0f, z: 0.0f)
                                point(x: -1.0f, y: -1.0f, z: 0.0f)
                                point(x: 1.0f, y: -1.0f, z: 0.0f)
                            }
                            colors {
                                color(r: 1.0f, g: 0.0f, b: 0.0f)
                                color(r: 0.0f, g: 1.0f, b: 0.0f)
                                color(r: 0.0f, g: 0.0f, b: 1.0f)
                            }
                        }
                    }
                }
                transform2 = transformGroup(translation: new Vector3f(3.0f, 0.0f, 0.0f), capability: TransformGroup.ALLOW_TRANSFORM_WRITE) {
                    shape3D {
                        quadArray(numVertices: 4, format: GeometryArray.COORDINATES | GeometryArray.COLOR_3) {
                            vertices {
                                point(x: -1.0f, y: -1.0f, z: 0.0f)
                                point(x: 1.0f, y: -1.0f, z: 0.0f)
                                point(x: 1.0f, y: 1.0f, z: 0.0f)
                                point(x: -1.0f, y: 1.0f, z: 0.0f)
                            }
                            colors {
                                color(r: 0.5f, g: 0.5f, b: 1.0f)
                                color(r: 0.5f, g: 0.5f, b: 1.0f)
                                color(r: 0.5f, g: 0.5f, b: 1.0f)
                            }
                        }
                    }
                }

            }
            viewTransformGroup(translation: new Vector3f(1.5f, 0.0f, 6.5f))
        }

        animator.start()
    }

    public void timingEvent(float fraction) {
        rtri += 0.1f
        rquad -= 0.15f
        def quadTrans = new Transform3D()
        def rot = new Transform3D()
        rot.rotX(rquad)
        quadTrans.setTranslation(new Vector3f(3.0f, 0.0f, 0.0f))
        quadTrans.mul(rot)
        //quadTrans.rotX(rquad)

        def triTrans = new Transform3D()
        triTrans.rotY(rtri)

        transform2.setTransform(quadTrans)
        transform.setTransform(triTrans)
    }

    void setupAnimation() {

    }
    void setupScene() {
        canvas3D.addKeyListener(keyListener)

        frame = swing.frame(size: [400, 200], title: 'Lesson 04: Rotation', defaultCloseOperation: WindowConstants.EXIT_ON_CLOSE) {
            widget(canvas3D)
        }

        swing.doLater() {
            frame.show()
        }
    }

    public static void main(args) {
        def demo = new Lesson04()
        demo.createGeometryAndView()
        demo.setupScene()
    }

    public void begin() {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void end() {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void repeat() {
        //To change body of implemented methods use File | Settings | File Templates.
    }
}